import {createMultipleUsers, createUser} from '../support/helpers';
import {randomFirstName, randomLastName, randomString, randomUserData, type UserData} from '../support/rand';
import {checkComboBoxOptions} from '../support/common';

const school = 'school1';

const submitUserSearch = (user?: UserData): void => {
  cy.url().should('eq', `${Cypress.config().baseUrl}/`);
  cy.get('input[name="schoolSelection"]').type(school).type('{enter}');
  cy.get('[name="quickSearch"]')
    .clear();
  if (user?.username) {
    cy.get('[name="quickSearch"]')
      .type(user.username);
  }
  const interceptKey = `getUsers-${randomString(16)}`;
  cy.intercept('GET', '/ucsschool/bff-users/v1/users*').as(interceptKey);
  cy.contains('.searchForm__buttons .uButton', 'Suchen').click();
  cy.get('.uStandbyWrapper').should('be.visible');
  cy.wait(`@${interceptKey}`).its('response.statusCode').should('eq', 200);
  cy.get('.uStandbyWrapper').should('not.exist');
};

const _findUserRowIfExists = (user: UserData): Cypress.Chainable<JQuery<HTMLElement>> => cy.get('.uGrid').then($grid => {
  const $node = $grid.find(`:contains(${user.username})`);
  if ($node.length) {
    return $node.closest('tr');
  }
  return $node;
}).then($node => {
  if ($node.length > 0) {
    // expect($node.find('td:nth-child(2)')).to.contain.text(user.firstname);
    // expect($node.find('td:nth-child(3)')).to.contain.text(user.lastname);
    expect($node.find('td:nth-child(5)')).to.contain.text(user.username);
  }
  return $node;
});

const _setPage = (page: number): Cypress.Chainable<boolean> => cy.get('.uGrid').then($grid => {
  if (page < 1) {
    return cy.wrap(false).then(() => false);
  }
  const $select = $grid.find('.uTable__pagination select[name="page"]');
  if ($select.length === 0) {
    const result = page === 1;
    return cy.wrap(result).then(() => result);
  }
  const $lastOption = $select.find('option:last-child');
  const lastPage = $lastOption.length ? parseInt($lastOption.val() as string, 10) : 1;
  if (page > lastPage) {
    return cy.wrap(false).then(() => false);
  }
  const newValue = String(page);
  if ($select.val() === newValue) {
    return cy.wrap(true).then(() => true);
  }
  if ($select.val() === null) {
    // The select is disabled. This happens if there is no data in the table.
    //  We will return true iff page #1 was requested (as that is the only
    //  available option in such a case)
    return cy.wrap(newValue === '1').then(result => result);
  }
  cy.wrap($select).select(String(page));
  // The next line is flaky: we are waiting a tiny bit to make sure the table
  //  content will be re-rendered before moving on. It would be much nicer to
  //  have a [data-page="X"] attribute on the table DOM to avoid having to
  //  wait for the table to reload like this.
  // eslint-disable-next-line cypress/no-unnecessary-waiting
  return cy.wait(100).then(() => true);
});

const _determineCurrentPage = (): Cypress.Chainable<number> => cy.get('.uGrid').then($grid => {
  const $select = $grid.find('.uTable__pagination select[name="page"]');
  if ($select.length === 0) {
    return 1;
  }
  return parseInt($select.val() as string, 10);
});

const _determineLastPage = (): Cypress.Chainable<number> => cy.get('.uGrid').then($grid => {
  const $option = $grid.find('.uTable__pagination select[name="page"] option:last-child');
  if ($option.length === 0) {
    return 1;
  }
  return parseInt($option.val() as string, 10);
});

const _resetPaging = (position: 'first' | 'last' = 'first'): Cypress.Chainable => {
  if (position === 'first') {
    return _setPage(1);
  }
  return _determineLastPage().then(page => _setPage(page));
};

const _adjustPage = (diff: number): Cypress.Chainable<boolean> => _determineCurrentPage().then(page => _setPage(page + diff));

const _findUserRowByPaging = (user: UserData, direction: 1 | -1): Cypress.Chainable<JQuery<HTMLElement>> => _findUserRowIfExists(user).then($node => {
  if ($node.length) {
    return cy.wrap($node);
  }
  return _adjustPage(direction).then(foundNextPage => {
    if (foundNextPage) {
      return _findUserRowByPaging(user, direction);
    }
    return _findUserRowIfExists(user);
  });
});

const _findUserRowByPagingForward = (user: UserData): Cypress.Chainable<JQuery<HTMLElement>> => _findUserRowByPaging(user, 1);

const _findUserRowByPagingBackward = (user: UserData): Cypress.Chainable<JQuery<HTMLElement>> => _findUserRowByPaging(user, -1);

const findUserRow = (user: UserData, options?: {assertExists?: boolean; searchForward?: boolean}): Cypress.Chainable<JQuery<HTMLElement>> => _resetPaging(options?.searchForward ? 'first' : 'last').then(success => {
  expect(success).to.be.equal(true);
  if (options?.searchForward) {
    return _findUserRowByPagingForward(user);
  }
  return _findUserRowByPagingBackward(user);
}).then($row => {
  if (options?.assertExists) {
    expect($row).to.have.length.above(0, `Finding user "${user.username}"`);

    // Align to 'end' of scrollable area since the table header can overlap the row with
    // `block: 'start' (which is the default)
    $row[0].scrollIntoView({block: 'end'});
  }
  return $row;
});

const testClasses = [
  {
    school: 'school2',
    name: randomString(6),
  },
];

const getKelvinToken = (): Cypress.Chainable<Promise<string>> => cy.request({
  method: 'POST',
  url: 'http://backup1.school.test/ucsschool/kelvin/token',
  form: true,
  body: {
    // eslint-disable-next-line camelcase
    grant_type: 'password',
    username: 'Administrator',
    password: 'univention',
  },
}).then(response => response.body.access_token);

before(() => {
  getKelvinToken().then(token => {
    for (const claz of testClasses) {
      cy.request({
        method: 'POST',
        url: 'http://backup1.school.test/ucsschool/kelvin/v1/classes/',
        headers: {
          Authorization: `Bearer ${token}`,
        },
        body: {
          // eslint-disable-next-line camelcase
          udm_properties: {},
          name: claz.name,
          school: `https://backup1.school.test/ucsschool/kelvin/v1/schools/${claz.school}`,
          description: '',
          users: [],
          // eslint-disable-next-line camelcase
          create_share: false,
        },
      });
    }
  });
});

after(() => {
  getKelvinToken().then(token => {
    for (const claz of testClasses) {
      cy.request({
        method: 'DELETE',
        url: `http://backup1.school.test/ucsschool/kelvin/v1/classes/${claz.school}/${claz.name}`,
        headers: {
          Authorization: `Bearer ${token}`,
        },
        failOnStatusCode: false,
      });
    }
  });
});

describe('User module - ListView - search form', () => {
  it('has correct initial values selected', () => {
    cy.login();
    cy.intercept('GET', '/ucsschool/bff-users/v1/pageconf/listView').as('listView');
    cy.visit('/');
    cy.wait('@listView').then(({response}) => {
      expect(response?.statusCode).equal(200);
      cy.get('input[name="schoolSelection"]').should('have.value', response?.body.searchFormOptions.schools[0].label);
      cy.get('input[name="groupSelection"]').should('have.value', '---');
      cy.get('input[name="stateSelection"]').should('have.value', 'Aktiviert'); // TODO translations
      cy.get('input[name="wlanSelection"]').should('have.value', '---');
      cy.get('input[name="roleSelection"]').should('have.value', '---');
      cy.get('input[name="sourceUID"]').should('have.value', '---');
      cy.get('input[name="quickSearch"]').should('have.value', '');
    });
  });

  it('has expected combobox options', () => {
    cy.login();
    cy.intercept('GET', '/ucsschool/bff-users/v1/pageconf/listView').as('listView');
    cy.intercept('GET', '/ucsschool/bff-users/v1/school_class?school=*').as('schoolClasses');
    cy.intercept('GET', '/ucsschool/bff-users/v1/workgroup?school=*').as('workGroups');
    cy.visit('/');
    cy.wait('@listView').then(({response}) => {
      expect(response?.statusCode).equal(200);
      // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
      const searchFormOptions = response!.body.searchFormOptions;

      const initialSchool = searchFormOptions.schools[0];
      cy.wait(['@schoolClasses', '@workGroups']).then(interception => {
        expect(interception[0].response?.statusCode).equal(200);
        expect(interception[1].response?.statusCode).equal(200);
        expect(interception[0].request.query.school).to.equal(initialSchool.value);
        expect(interception[1].request.query.school).to.equal(initialSchool.value);
        // eslint-disable-next-line no-unused-expressions,@typescript-eslint/no-unused-expressions
        expect(interception[0].response).to.not.be.undefined;
        // eslint-disable-next-line no-unused-expressions,@typescript-eslint/no-unused-expressions
        expect(interception[1].response).to.not.be.undefined;
        const options = [
          {
            value: '',
            label: '---',
          },
          // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
          ...interception[0].response!.body.map((option: {label: string}) => {
            option.label = `${option.label} (Schulklasse)`;
            return option;
          }),
          // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
          ...interception[1].response!.body.map((option: {label: string}) => {
            option.label = `${option.label} (Arbeitsgruppe)`;
            return option;
          }),
        ];
        checkComboBoxOptions('groupSelection', options);
      });

      checkComboBoxOptions('schoolSelection', searchFormOptions.schools);
      const activationStates = [
        {value: '',
          label: '---'}, ...searchFormOptions.activationStates,
      ];
      checkComboBoxOptions('stateSelection', activationStates);
      const networkAccess = [
        {value: '',
          label: '---'}, ...searchFormOptions.networkAccess,
      ];
      checkComboBoxOptions('wlanSelection', networkAccess);
      const roles = [
        {value: '',
          label: '---'}, ...searchFormOptions.roles,
      ];
      checkComboBoxOptions('roleSelection', roles);
      const sourceUIDs = [
        {
          label: '---',
          value: '',
        },
        {
          label: 'DiViS',
          value: 'DiViS',
        },
        {
          label: 'Manuell',
          value: 'Manuell',
        },
      ];
      checkComboBoxOptions('sourceUID', sourceUIDs);
    });
  });

  it('has correct values after reset', () => {
    cy.login();
    cy.intercept('GET', '/ucsschool/bff-users/v1/pageconf/listView').as('listView');
    cy.intercept('GET', '/ucsschool/bff-users/v1/school_class?school=*').as('schoolClasses');
    cy.intercept('GET', '/ucsschool/bff-users/v1/workgroup?school=*').as('workGroups');
    cy.visit('/');
    cy.wait('@listView').then(({response}) => {
      expect(response?.statusCode).equal(200);
      // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
      const searchFormOptions = response!.body.searchFormOptions;
      cy.wait(['@schoolClasses', '@workGroups']).then(() => {
        // FIXME don't use hard coded `DEMOSCHOOL`
        cy.get('input[name="schoolSelection"]').type('DEMOSCHOOL').type('{enter}');
        cy.wait(['@schoolClasses', '@workGroups']).then(interception => {
          expect(interception[0].response?.statusCode).equal(200);
          expect(interception[1].response?.statusCode).equal(200);

          const options = [
            {
              value: '',
              label: '---',
            },
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            ...interception[0].response!.body.map((option: {label: string}) => {
              option.label = `${option.label} (Schulklasse)`;
              return option;
            }),
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            ...interception[1].response!.body.map((option: {label: string}) => {
              option.label = `${option.label} (Arbeitsgruppe)`;
              return option;
            }),
          ];
          cy.get('input[name="groupSelection"]').type(options[1].label).type('{enter}');
          cy.get('input[name="wlanSelection"]').type(searchFormOptions.networkAccess[1].label).type('{enter}');
          cy.get('input[name="roleSelection"]').type(searchFormOptions.roles[1].label).type('{enter}');
          cy.get('input[name="sourceUID"]').type('DiViS').type('{enter}');
          cy.get('input[name="quickSearch"]').type('foobar');
          cy.contains('.uButton', 'Zurücksetzen').click();
          cy.get('input[name="schoolSelection"]').should('have.value', 'DEMOSCHOOL');
          cy.get('input[name="groupSelection"]').should('have.value', '---');
          cy.get('input[name="stateSelection"]').should('have.value', '---');
          cy.get('input[name="wlanSelection"]').should('have.value', '---');
          cy.get('input[name="roleSelection"]').should('have.value', '---');
          cy.get('input[name="sourceUID"]').should('have.value', '---');
          cy.get('input[name="quickSearch"]').should('have.value', '');
        });
      });
    });
  });

});

describe('User module', () => {

  it('creates and deletes users correctly', () => {
    cy.login();
    createUser().then(userData => {
      cy.get('.uTable__table tbody tr')
        .should('have.length', 0);
      cy.contains('.uGrid__header .uButton', 'Löschen')
        .should('be.disabled');
      submitUserSearch(userData);
      findUserRow(userData, {assertExists: true})
        .find('td:first-child input').click({scrollBehavior: 'center'});
      cy.intercept('DELETE', `ucsschool/bff-users/v1/users/${userData.username}`).as('deleteUser');
      cy.contains('.uGrid__header .uButton', 'Löschen')
        .should('not.be.disabled')
        .click();
      cy.get('.uConfirmDialog .uButton--primary')
        .click();
      cy.wait('@deleteUser').its('response.statusCode').should('eq', 204);
      cy.get('.uTable__table tbody tr')
        .should('have.length', 0);
      submitUserSearch(userData);
      cy.get('.uTable__table tbody tr')
        .should('have.length', 0);
    });
  });

  it('deletes multiple users correctly', () => {
    cy.login();
    createMultipleUsers(2).then(users => {
      submitUserSearch();
      findUserRow(users[0], {assertExists: true})
        .find('td:first-child input').click({scrollBehavior: 'center'});
      findUserRow(users[1], {assertExists: true})
        .find('td:first-child input').click({scrollBehavior: 'center'});

      cy.intercept('DELETE', `ucsschool/bff-users/v1/users/${users[0].username}`).as('deleteUser1');
      cy.intercept('DELETE', `ucsschool/bff-users/v1/users/${users[1].username}`).as('deleteUser2');
      cy.contains('.uGrid__header .uButton', 'Löschen')
        .should('not.be.disabled')
        .click();
      cy.get('.uConfirmDialog .uButton--primary')
        .click();
      cy.wait('@deleteUser1').its('response.statusCode').should('eq', 204);
      cy.wait('@deleteUser2').its('response.statusCode').should('eq', 204);
      submitUserSearch(users[0]);
      findUserRow(users[0]).should('have.length', 0);
      submitUserSearch(users[1]);
      findUserRow(users[1]).should('have.length', 0);
    });
  });

  it('edits user correctly', () => {
    cy.login();
    const originalUserData = randomUserData();
    const editedUserData = randomUserData({nickname: true});
    createUser(originalUserData);
    submitUserSearch(originalUserData);
    cy.contains('.uGrid__header .uButton', 'Bearbeiten')
      .should('be.disabled');
    findUserRow(originalUserData, {assertExists: true})
      .find('td:first-child input')
      .click({scrollBehavior: 'center'});
    cy.intercept('GET', `ucsschool/bff-users/v1/users/${originalUserData.username}`).as('getUser');
    cy.contains('.uGrid__header .uButton', 'Bearbeiten')
      .should('not.be.disabled')
      .click();
    cy.wait('@getUser').its('response.statusCode').should('eq', 200);
    cy.url().should('eq', `${Cypress.config().baseUrl}/edit/${originalUserData.username}`);
    cy.get('input[name="firstname"]')
      .clear({scrollBehavior: 'center'})
      .type(editedUserData.firstname, {scrollBehavior: 'center'});
    cy.get('input[name="lastname"]')
      .clear({scrollBehavior: 'center'})
      .type(editedUserData.lastname, {scrollBehavior: 'center'});
    if (editedUserData.nickname) {
      cy.get('input[name="divisNickname"]')
        .clear({scrollBehavior: 'center'})
        .type(editedUserData.nickname, {scrollBehavior: 'center'});
    }
    cy.get('input[name="name"]')
      .clear({scrollBehavior: 'center'})
      .type(editedUserData.username, {scrollBehavior: 'center'});
    const url = new URL(Cypress.config().baseUrl as string);
    cy.intercept({
      method: 'PATCH',
      hostname: url.hostname,
    }).as('patch');
    cy.contains('.listView__header__buttons .uButton', 'Speichern')
      .click();
    cy.wait('@patch').its('response.statusCode').should('eq', 204);
    cy.get('.uStandbyWrapper').should('have.length', 1);
    cy.get('.uStandbyWrapper').should('have.length', 0);
    submitUserSearch();
    findUserRow(originalUserData).then($row => {
      cy.wrap($row).should('have.length', 0);
    });
    findUserRow(editedUserData, {assertExists: true}).then($row => {
      if (editedUserData.nickname) {
        cy.wrap($row.find('td:nth-child(4)')).should('contain.text', editedUserData.nickname);
      }
    });
  });

  it('shows correct username and email in modal when creating user', () => {
    cy.login();
    createUser({
      firstname: randomFirstName(),
      lastname: randomLastName(),
      username: null,
      nickname: null,
      birthday: null,
      email: null,
      role: 'student',
    });
    createUser({
      firstname: randomFirstName(),
      lastname: randomLastName(),
      username: randomString(8),
      nickname: null,
      birthday: null,
      email: null,
      role: 'student',
    });
  });

  it.skip('actions are correctly disabled/not disabled based on role', () => {

    interface ExpectedGlobalActionStates {
      add: string;
    }
    interface ExpectedContextActionStates {
      edit: string;
      delete: string;
      setPassword: string;
      setWLANPassword: string;
      activateUser: string;
      deactivateUser: string;
      activateWifi: string;
      deactivateWifi: string;
      exportCsv: string;
    }
    const testGlobalActions = (expectedStates: ExpectedGlobalActionStates): void => {
      cy.contains('.uGrid__header .uButton', 'Hinzufügen').should(expectedStates.add);
    };
    const testContextActions = (subject: UserData, expectedStates: ExpectedContextActionStates): void => {
      const tr = findUserRow(subject, {assertExists: true});

      cy.contains('.uGrid__header .uButton', 'Mehr').as('moreButton');

      cy.contains('.uGrid__header .uButton', 'Bearbeiten').should('be.disabled');
      cy.contains('.uGrid__header .uButton', 'Löschen').should('be.disabled');
      cy.get('@moreButton').should('not.be.disabled');
      cy.get('@moreButton').click();
      cy.contains('.uGrid__menuAction .uButton', 'Passwörter setzen').should('be.disabled');
      cy.contains('.uGrid__menuAction .uButton', 'WLAN-Passwörter setzen').should('be.disabled');
      cy.contains('.uGrid__menuAction .uButton', 'Benutzer aktivieren').should('be.disabled');
      cy.contains('.uGrid__menuAction .uButton', 'Benutzer deaktivieren').should('be.disabled');
      cy.contains('.uGrid__menuAction .uButton', 'Wifi Zugang aktivieren').should('be.disabled');
      cy.contains('.uGrid__menuAction .uButton', 'Wifi Zugang deaktivieren').should('be.disabled');
      cy.contains('.uGrid__menuAction .uButton', 'In CSV-Datei exportieren').should('be.disabled');
      cy.get('@moreButton').click();

      tr.find('td:first-child input').click({scrollBehavior: 'center'});

      cy.contains('.uGrid__header .uButton', 'Bearbeiten').should(expectedStates.edit);
      cy.contains('.uGrid__header .uButton', 'Löschen').should(expectedStates.delete);
      cy.get('@moreButton').should('not.be.disabled');
      cy.get('@moreButton').click();
      cy.contains('.uGrid__menuAction .uButton', 'Passwörter setzen').should(expectedStates.setPassword);
      cy.contains('.uGrid__menuAction .uButton', 'WLAN-Passwörter setzen').should(expectedStates.setWLANPassword);
      cy.contains('.uGrid__menuAction .uButton', 'Benutzer aktivieren').should(expectedStates.activateUser);
      cy.contains('.uGrid__menuAction .uButton', 'Benutzer deaktivieren').should(expectedStates.deactivateUser);
      cy.contains('.uGrid__menuAction .uButton', 'Wifi Zugang aktivieren').should(expectedStates.activateWifi);
      cy.contains('.uGrid__menuAction .uButton', 'Wifi Zugang deaktivieren').should(expectedStates.deactivateWifi);
      cy.contains('.uGrid__menuAction .uButton', 'In CSV-Datei exportieren').should(expectedStates.exportCsv);
      cy.get('@moreButton').click();
    };
    cy.login();

    const student = randomUserData();
    student.role = 'student';
    createUser(student);

    const student2 = randomUserData();
    student2.role = 'student';
    createUser(student2);

    const teacher = randomUserData();
    teacher.role = 'teacher';
    createUser(teacher);

    const teacher2 = randomUserData();
    teacher2.role = 'teacher';
    createUser(teacher2);

    const schoolAdmin = randomUserData();
    schoolAdmin.role = 'schoolAdmin';
    createUser(schoolAdmin);

    const schoolAdmin2 = randomUserData();
    schoolAdmin2.role = 'schoolAdmin';
    createUser(schoolAdmin2);

    const technicalAdmin = randomUserData();
    technicalAdmin.role = 'technicalAdmin';
    createUser(technicalAdmin);

    const technicalAdmin2 = randomUserData();
    technicalAdmin2.role = 'technicalAdmin';
    createUser(technicalAdmin2);


    // test logged in as student
    cy.log('###test as student');
    cy.login({
      username: student.username,
      password: 'univention',
    });
    testGlobalActions({
      add: 'be.disabled',
    });
    submitUserSearch(student);
    cy.log('..against student');
    testContextActions(student, {
      edit: 'be.disabled',
      delete: 'be.disabled',
      setPassword: 'be.disabled',
      setWLANPassword: 'be.disabled',
      activateUser: 'be.disabled',
      deactivateUser: 'be.disabled',
      activateWifi: 'be.disabled',
      deactivateWifi: 'be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(student2);
    findUserRow(student2).should('have.length', 0);
    submitUserSearch(teacher);
    findUserRow(teacher).should('have.length', 0);
    submitUserSearch(schoolAdmin);
    findUserRow(schoolAdmin).should('have.length', 0);
    submitUserSearch(technicalAdmin);
    findUserRow(technicalAdmin).should('have.length', 0);


    // test logged in as teacher
    cy.log('###test as teacher');
    cy.login({
      username: teacher.username,
      password: 'univention',
    });
    testGlobalActions({
      add: 'be.disabled',
    });
    submitUserSearch(student);
    cy.log('..against student');
    testContextActions(student, {
      edit: 'not.be.disabled',
      delete: 'be.disabled',
      setPassword: 'not.be.disabled',
      setWLANPassword: 'not.be.disabled',
      activateUser: 'be.disabled',
      deactivateUser: 'be.disabled',
      activateWifi: 'not.be.disabled',
      deactivateWifi: 'not.be.disabled',
      exportCsv: 'be.disabled',
    });
    submitUserSearch(teacher);
    cy.log('..against teacher (self)');
    testContextActions(teacher, {
      edit: 'not.be.disabled',
      delete: 'be.disabled',
      setPassword: 'be.disabled',
      setWLANPassword: 'be.disabled',
      activateUser: 'be.disabled',
      deactivateUser: 'be.disabled',
      activateWifi: 'be.disabled',
      deactivateWifi: 'be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(teacher2);
    cy.log('..against teacher');
    testContextActions(teacher2, {
      edit: 'not.be.disabled',
      delete: 'be.disabled',
      setPassword: 'be.disabled',
      setWLANPassword: 'be.disabled',
      activateUser: 'be.disabled',
      deactivateUser: 'be.disabled',
      activateWifi: 'be.disabled',
      deactivateWifi: 'be.disabled',
      exportCsv: 'be.disabled',
    });
    submitUserSearch(schoolAdmin);
    cy.log('..against schooladmin');
    testContextActions(schoolAdmin, {
      edit: 'be.disabled',
      delete: 'be.disabled',
      setPassword: 'be.disabled',
      setWLANPassword: 'be.disabled',
      activateUser: 'be.disabled',
      deactivateUser: 'be.disabled',
      activateWifi: 'be.disabled',
      deactivateWifi: 'be.disabled',
      exportCsv: 'be.disabled',
    });
    submitUserSearch(technicalAdmin);
    findUserRow(technicalAdmin).should('have.length', 0);

    // test logged in as school admin
    cy.log('###test as schooladmin');
    cy.login({
      username: schoolAdmin.username,
      password: 'univention',
    });
    testGlobalActions({
      add: 'not.be.disabled',
    });
    submitUserSearch(student);
    cy.log('..against student');
    testContextActions(student, {
      edit: 'not.be.disabled',
      delete: 'not.be.disabled',
      setPassword: 'not.be.disabled',
      setWLANPassword: 'not.be.disabled',
      activateUser: 'not.be.disabled',
      deactivateUser: 'not.be.disabled',
      activateWifi: 'not.be.disabled',
      deactivateWifi: 'not.be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(teacher);
    cy.log('..against teacher');
    testContextActions(teacher, {
      edit: 'not.be.disabled',
      delete: 'not.be.disabled',
      setPassword: 'not.be.disabled',
      setWLANPassword: 'not.be.disabled',
      activateUser: 'not.be.disabled',
      deactivateUser: 'not.be.disabled',
      activateWifi: 'not.be.disabled',
      deactivateWifi: 'not.be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(schoolAdmin);
    cy.log('..against schooladmin (self)');
    testContextActions(schoolAdmin, {
      edit: 'be.disabled',
      delete: 'be.disabled',
      setPassword: 'be.disabled',
      setWLANPassword: 'be.disabled',
      activateUser: 'be.disabled',
      deactivateUser: 'be.disabled',
      activateWifi: 'be.disabled',
      deactivateWifi: 'be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(schoolAdmin2);
    cy.log('..against schooladmin');
    testContextActions(schoolAdmin2, {
      edit: 'be.disabled',
      delete: 'be.disabled',
      setPassword: 'be.disabled',
      setWLANPassword: 'be.disabled',
      activateUser: 'be.disabled',
      deactivateUser: 'be.disabled',
      activateWifi: 'be.disabled',
      deactivateWifi: 'be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(technicalAdmin);
    findUserRow(technicalAdmin).should('have.length', 0);

    // test logged in as technical admin
    cy.log('###test as technicaladmin');
    cy.login({
      username: technicalAdmin.username,
      password: 'univention',
    });
    testGlobalActions({
      add: 'not.be.disabled',
    });
    submitUserSearch(student);
    cy.log('..against student');
    testContextActions(student, {
      edit: 'not.be.disabled',
      delete: 'not.be.disabled',
      setPassword: 'not.be.disabled',
      setWLANPassword: 'not.be.disabled',
      activateUser: 'not.be.disabled',
      deactivateUser: 'not.be.disabled',
      activateWifi: 'not.be.disabled',
      deactivateWifi: 'not.be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(teacher);
    cy.log('..against teacher');
    testContextActions(teacher, {
      edit: 'not.be.disabled',
      delete: 'not.be.disabled',
      setPassword: 'not.be.disabled',
      setWLANPassword: 'not.be.disabled',
      activateUser: 'not.be.disabled',
      deactivateUser: 'not.be.disabled',
      activateWifi: 'not.be.disabled',
      deactivateWifi: 'not.be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(schoolAdmin);
    cy.log('..against schooladmin');
    testContextActions(schoolAdmin, {
      edit: 'not.be.disabled',
      delete: 'not.be.disabled',
      setPassword: 'not.be.disabled',
      setWLANPassword: 'not.be.disabled',
      activateUser: 'not.be.disabled',
      deactivateUser: 'not.be.disabled',
      activateWifi: 'not.be.disabled',
      deactivateWifi: 'not.be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(technicalAdmin);
    cy.log('..against technicaladmin (self)');
    testContextActions(technicalAdmin, {
      edit: 'not.be.disabled',
      delete: 'not.be.disabled',
      setPassword: 'not.be.disabled',
      setWLANPassword: 'not.be.disabled',
      activateUser: 'not.be.disabled',
      deactivateUser: 'not.be.disabled',
      activateWifi: 'not.be.disabled',
      deactivateWifi: 'not.be.disabled',
      exportCsv: 'not.be.disabled',
    });
    submitUserSearch(technicalAdmin2);
    cy.log('..against technicaladmin');
    testContextActions(technicalAdmin2, {
      edit: 'not.be.disabled',
      delete: 'not.be.disabled',
      setPassword: 'not.be.disabled',
      setWLANPassword: 'not.be.disabled',
      activateUser: 'not.be.disabled',
      deactivateUser: 'not.be.disabled',
      activateWifi: 'not.be.disabled',
      deactivateWifi: 'not.be.disabled',
      exportCsv: 'not.be.disabled',
    });
  });
});
